@@ -7,7 +7,10 @@ use {
     itertools::izip,
     log::*,
     solana_client::thin_client::{create_client, ThinClient},
-    solana_core::validator::{Validator, ValidatorConfig, ValidatorStartProgress},
+    solana_core::{
+        tower_storage::FileTowerStorage,
+        validator::{Validator, ValidatorConfig, ValidatorStartProgress},
+    },
     solana_gossip::{
         cluster_info::{Node, VALIDATOR_PORT_RANGE},
         contact_info::ContactInfo,
@@ -50,7 +53,6 @@ use {
     },
 };
 
-#[derive(Debug)]
 pub struct ClusterConfig {
     /// The validator config that should be applied to every node in the cluster
     pub validator_configs: Vec<ValidatorConfig>,
@@ -145,6 +147,11 @@ impl LocalCluster {
                 .iter()
                 .zip(&config.node_stakes)
                 .filter_map(|((node_keypair, in_genesis), stake)| {
+                    info!(
+                        "STARTING LOCAL CLUSTER: key {} has {} stake",
+                        node_keypair.pubkey(),
+                        stake
+                    );
                     if *in_genesis {
                         Some((
                             ValidatorVoteKeypairs {
@@ -208,13 +215,14 @@ impl LocalCluster {
         let mut leader_config = safe_clone_config(&config.validator_configs[0]);
         leader_config.rpc_addrs = Some((leader_node.info.rpc, leader_node.info.rpc_pubsub));
         leader_config.account_paths = vec![leader_ledger_path.join("accounts")];
+        leader_config.tower_storage = Arc::new(FileTowerStorage::new(leader_ledger_path.clone()));
         let leader_keypair = Arc::new(Keypair::from_bytes(&leader_keypair.to_bytes()).unwrap());
         let leader_vote_keypair =
             Arc::new(Keypair::from_bytes(&leader_vote_keypair.to_bytes()).unwrap());
 
         let leader_server = Validator::new(
             leader_node,
-            &leader_keypair,
+            leader_keypair.clone(),
             &leader_ledger_path,
             &leader_vote_keypair.pubkey(),
             Arc::new(RwLock::new(vec![leader_vote_keypair.clone()])),
@@ -405,10 +413,11 @@ impl LocalCluster {
         let mut config = safe_clone_config(validator_config);
         config.rpc_addrs = Some((validator_node.info.rpc, validator_node.info.rpc_pubsub));
         config.account_paths = vec![ledger_path.join("accounts")];
+        config.tower_storage = Arc::new(FileTowerStorage::new(ledger_path.clone()));
         let voting_keypair = voting_keypair.unwrap();
         let validator_server = Validator::new(
             validator_node,
-            &validator_keypair,
+            validator_keypair.clone(),
             &ledger_path,
             &voting_keypair.pubkey(),
             Arc::new(RwLock::new(vec![voting_keypair.clone()])),
@@ -513,8 +522,8 @@ impl LocalCluster {
         lamports: u64,
     ) -> u64 {
         trace!("getting leader blockhash");
-        let (blockhash, _fee_calculator, _last_valid_slot) = client
-            .get_recent_blockhash_with_commitment(CommitmentConfig::processed())
+        let (blockhash, _) = client
+            .get_latest_blockhash_with_commitment(CommitmentConfig::processed())
             .unwrap();
         let mut tx = system_transaction::transfer(source_keypair, dest_pubkey, lamports, blockhash);
         info!(
@@ -574,7 +583,7 @@ impl LocalCluster {
                 &[from_account.as_ref(), vote_account],
                 message,
                 client
-                    .get_recent_blockhash_with_commitment(CommitmentConfig::processed())
+                    .get_latest_blockhash_with_commitment(CommitmentConfig::processed())
                     .unwrap()
                     .0,
             );
@@ -602,7 +611,7 @@ impl LocalCluster {
                 &[from_account.as_ref(), &stake_account_keypair],
                 message,
                 client
-                    .get_recent_blockhash_with_commitment(CommitmentConfig::processed())
+                    .get_latest_blockhash_with_commitment(CommitmentConfig::processed())
                     .unwrap()
                     .0,
             );
@@ -742,9 +751,11 @@ impl Cluster for LocalCluster {
         let validator_info = &cluster_validator_info.info;
         cluster_validator_info.config.account_paths =
             vec![validator_info.ledger_path.join("accounts")];
+        cluster_validator_info.config.tower_storage =
+            Arc::new(FileTowerStorage::new(validator_info.ledger_path.clone()));
         let restarted_node = Validator::new(
             node,
-            &validator_info.keypair,
+            validator_info.keypair.clone(),
             &validator_info.ledger_path,
             &validator_info.voting_keypair.pubkey(),
             Arc::new(RwLock::new(vec![validator_info.voting_keypair.clone()])),
@@ -781,3 +792,37 @@ impl Drop for LocalCluster {
         self.close();
     }
 }
+
+#[cfg(test)]
+mod test {
+    use {super::*, solana_sdk::epoch_schedule::MINIMUM_SLOTS_PER_EPOCH};
+
+    #[test]
+    fn test_local_cluster_start_and_exit() {
+        solana_logger::setup();
+        let num_nodes = 1;
+        let cluster =
+            LocalCluster::new_with_equal_stakes(num_nodes, 100, 3, SocketAddrSpace::Unspecified);
+        assert_eq!(cluster.validators.len(), num_nodes);
+    }
+
+    #[test]
+    fn test_local_cluster_start_and_exit_with_config() {
+        solana_logger::setup();
+        const NUM_NODES: usize = 1;
+        let mut config = ClusterConfig {
+            validator_configs: make_identical_validator_configs(
+                &ValidatorConfig::default(),
+                NUM_NODES,
+            ),
+            node_stakes: vec![3; NUM_NODES],
+            cluster_lamports: 100,
+            ticks_per_slot: 8,
+            slots_per_epoch: MINIMUM_SLOTS_PER_EPOCH as u64,
+            stakers_slot_offset: MINIMUM_SLOTS_PER_EPOCH as u64,
+            ..ClusterConfig::default()
+        };
+        let cluster = LocalCluster::new(&mut config, SocketAddrSpace::Unspecified);
+        assert_eq!(cluster.validators.len(), NUM_NODES);
+    }
+}
